@@ -93,6 +93,9 @@ type token =
   | LBRACKETGREATER
   | LBRACKETPERCENT
   | LBRACKETPERCENTPERCENT
+  | LBRACKETAT
+  | LBRACKETATAT
+  | LBRACKETATATAT
   | LESS
   | LESSMINUS
   | LET
