@@ -298,10 +298,16 @@ let unwind_while f path =
   | { kind } as h :: p when f kind -> Some (aux h p)
   | _ -> None
 
+let top_kind = function
+  | KStruct|KSig|KParen|KBegin|KObject|KExtendedItem _ -> true
+  | _ -> false
+
+let stritem_kind = function
+  | KModule|KVal|KLet|KExternal|KType|KException|KOpen|KInclude -> true
+  | _ -> false
+
 (* Unwind the struct/sig top *)
-let unwind_top =
-  unwind (function KStruct|KSig|KParen|KBegin|KObject|KExtendedItem _-> true
-                 | _ -> false)
+let unwind_top = unwind top_kind
 
 (* Get the parent node *)
 let parent = function
@@ -415,7 +421,7 @@ let op_prio_align_indent config =
   in
   let is_monadop s =
     match String.sub s 0 (min 2 (String.length s)) with
-    | ">>" | ">|" | "@@" -> true
+    | ">>" | ">|" | "@@" | "@>" -> true
     | _ -> false
   in
   let is_monadop s =
@@ -803,10 +809,14 @@ let rec update_path config block stream tok =
   | LPAREN -> open_paren KParen block.path
   | LBRACKET | LBRACKETGREATER | LBRACKETLESS ->
       open_paren KBracket block.path
-  | LBRACKETPERCENT ->
+  | LBRACKETPERCENT | LBRACKETAT ->
       let path = before_append_atom block.path in
       append ~pad:4 (KExtendedExpr []) L path
-  | LBRACKETPERCENTPERCENT ->
+  | LBRACKETATAT ->
+      append ~pad:4 (KExtendedItem []) L
+        (unwind (function KBody k | k -> top_kind k || stritem_kind k)
+            block.path)
+  | LBRACKETPERCENTPERCENT | LBRACKETATATAT ->
       append ~pad:4 (KExtendedItem []) L (unwind_top block.path)
   | LBRACKETBAR -> open_paren KBracketBar block.path
   | LBRACE | LBRACELESS ->
@@ -1119,7 +1129,13 @@ let rec update_path config block stream tok =
         | {kind=KBar m}::{kind=KWith _; line}::_ when line = current_line ->
             (* Special case: don't respect match_clause when 'with X ->' is on
               a single line *)
-            append (KArrow m) L ~pad:config.i_base path
+            let pad =
+              if next_offset tok stream <> None then config.i_base
+              else match next_token stream with
+                | Some (MATCH|TRY|FUN|FUNCTION) -> 0
+                | _ -> config.i_base
+            in
+            append (KArrow m) L ~pad path
         | {kind=KWith m | KBar m} :: _ ->
             let pad =
               config.i_match_clause
@@ -1148,6 +1164,7 @@ let rec update_path config block stream tok =
       let unwind_to = function
         | KParen | KBegin | KBrace | KBracket | KBracketBar | KBody _
         | KExternal | KModule | KType | KLet | KLetIn | KException | KVal
+        | KBar KType
         | KStruct | KSig | KObject
         | KAnd(KModule|KType|KLet|KLetIn) -> true
         | _ -> false
@@ -1175,7 +1192,7 @@ let rec update_path config block stream tok =
                   extend (KExpr (prio_semi+1)) T ~pad:config.i_base p
               | None ->
                   make_infix tok block.path)
-         | {kind=KParen|KBegin|KBracket|KBracketBar|KBody _}::_ ->
+         | {kind=KParen|KBegin|KBracket|KBracketBar|KBody _|KBar KType}::_ ->
              make_infix tok block.path
          | {kind=KAnd kind | kind} as h::p ->
              let indent = match next_token stream, kind with
@@ -1192,7 +1209,7 @@ let rec update_path config block stream tok =
       in
       find_parent block.path
 
-  | COLONEQUAL ->
+  | COLONEQUAL | INFIXOP2 "+=" ->
       (match
         unwind_while (function KExpr _ | KType -> true | _ -> false) block.path
       with
@@ -1269,6 +1286,15 @@ let rec update_path config block stream tok =
            { h with pad = config.i_base } :: p
        | _ -> make_infix tok block.path)
 
+  | AMPERAMPER | BARBAR ->
+      (* back-indented when after if or when and not alone *)
+      let op_prio, _align, _indent = op_prio_align_indent config tok.token in
+      (match unwind_while (fun kind -> prio kind >= op_prio) block.path with
+       | Some ({kind=KExpr _; line}::{kind=KWhen|KIf; line=line_if}::_ as p)
+         when line = line_if && next_offset tok stream <> None ->
+           extend (KExpr op_prio) T ~pad:(-3) p
+       | _ -> make_infix tok block.path)
+
   | LESS ->
       if is_inside_type block.path then
         (* object type *)
@@ -1292,7 +1318,6 @@ let rec update_path config block stream tok =
 
   | LESSMINUS | COMMA | OR
   | AMPERSAND | INFIXOP0 _ | INFIXOP1 _
-  | AMPERAMPER | BARBAR
   | COLONCOLON | INFIXOP2 _ | PLUSDOT | PLUS | MINUSDOT | MINUS
   | INFIXOP3 _ | STAR | INFIXOP4 _
   | SHARP | AS | COLONGREATER
@@ -1455,10 +1480,14 @@ let rec update_path config block stream tok =
             let col = Path.indent block.path + Path.pad block.path in
             append (KComment (tok,col)) (A col) ~pad block.path)
 
+  |DOTDOT ->
+      (match block.path with
+       | {kind = KBody KType} :: p -> p
+       | _ -> append KUnknown L block.path)
+
   |VIRTUAL
   |REC
   |PRIVATE|EOF
-  |DOTDOT
   |BACKQUOTE|ILLEGAL_CHAR _ ->
       (* indent the token, but otherwise ignored *)
       append KUnknown L block.path
@@ -1574,8 +1603,8 @@ let is_clean t =
     t.path
 
 let is_at_top t = match t.path with
-  | [] | [{kind=KModule|KVal|KLet|KExternal|KType|KException
-               |KOpen|KInclude}] -> true
+  | [] -> true
+  | [{kind}] -> stritem_kind kind
   | _ -> false
 
 let is_declaration t = is_clean t && match t.path with
