@@ -1,4 +1,3 @@
-(* maizatulin *)
 type t =
   { last_trading : Week_date.Spec.t;
     first_notice : Week_date.Spec.t option;
@@ -7,4 +6,5 @@ type t =
     offset       : Week_date.Offset.t;
     (* n > 0 *)
     new_contract_expires_in_n_months : int
-  } with sexp, compare
+  }
+[@@deriving sexp, compare]
