@@ -8,7 +8,8 @@ let handle_query qs ~msg_client:_ =
   )
 ;;
 
-if _ then _
+if _ then
+  _
 else
   assert_branch_has_node branch node >>| fun () ->
   { t with node; floating }
